import { pageIterator } from "$lib/graphql/query";
import { client } from "./graphql";
import { PROMOTION_QUERY, USER_DETAILS_QUERY } from "./graphql/queries";
import fromAsync from "array-from-async";

type UserId = string;
type PromoCache = {
  lastUpdateMs: number;
  promotion: Set<UserId>;
};

const cache = new Map<number, PromoCache>();
// 1 hour
const CACHE_DURATION_MS = 1000 * 60 * 60;

async function cacheImages(users: Iterable<UserId>): Promise<void> {
  let count = 0;
  for (const user of users) {
    const response = await client.query(USER_DETAILS_QUERY, { idList: [user] });

    if (response.error) {
      throw response.error;
    }

    const photo = response.data?.page.nodes[0].photo;
    if (!photo?.url) continue;
    await fetch(photo.url, { method: "HEAD" });
    count++;
  }
  console.log("cached", count, "images");
}

// background task
async function cachePromotionImages(year: number) {
  const promo = cache.get(year);
  if (!promo) return;
  await cacheImages(promo.promotion);
}

async function fetchPromotion(year: number): Promise<PromoCache> {
  const array = await fromAsync(pageIterator(PROMOTION_QUERY, { year }));
  const users = array.map((node) => node.id);

  return {
    lastUpdateMs: new Date().getTime(),
    promotion: new Set(users),
  };
}

export async function getPromotion(year: number): Promise<Set<UserId>> {
  if (cache.has(year)) {
    const data = cache.get(year)!;
    if (new Date().getTime() - data.lastUpdateMs < CACHE_DURATION_MS)
      return data.promotion;
  }
  console.log("year", year, "not in cache, fetching");
  const freshData = await fetchPromotion(year);
  cache.set(year, freshData);
  cachePromotionImages(year);
  return freshData.promotion;
}

export async function* promotionIterator(
  min: number,
  max: number,
): AsyncGenerator<UserId, void, undefined> {
  for (let i = min; i <= max; i++) {
    const promotion = await getPromotion(i);
    yield* promotion;
  }
}

export function getPromotionRange(
  min: number,
  max = min,
): Promise<Set<UserId>> {
  return fromAsync(promotionIterator(min, max)).then((array) => new Set(array));
}

setInterval(() => {
  for (const year of cache.keys()) {
    cachePromotionImages(year);
  }
}, CACHE_DURATION_MS / 2);
